@@ -9,7 +9,7 @@ from cachetools import cached, TTLCache
 BASE_PATH = Path(__file__).resolve().parent
 
 # Importing the climate cat data - to be replaced by RAS data once available:
-   
+
 def get_climate_cats(climate_file:str) -> pd.DataFrame:
     '''Uses standard climate categorisation unless user selects the 10,000 year climate sequence,
     in which case this is used
@@ -53,10 +53,10 @@ def get_EWR_table(file_path:str = None) -> dict:
                             dtype='str', encoding='cp1252')
 
     if not file_path:
-        my_url = 'https://az3mdbastg001.blob.core.windows.net/mdba-public-data/NSWEWR_LIVE_DEV.csv'
+        my_url = "py_ewr/parameter_metadata/NSWEWR.csv"
         proxies={} # Populate with your proxy settings
-        s = requests.get(my_url, proxies=proxies).text
-        df = pd.read_csv(io.StringIO(s),
+        #s = requests.get(my_url, proxies=proxies).text
+        df = pd.read_csv(my_url,#io.StringIO(s),
             usecols=['PlanningUnitID', 'PlanningUnitName',  'LTWPShortName', 'CompliancePoint/Node', 'Gauge', 'Code', 'StartMonth',
                               'EndMonth', 'TargetFrequency', 'TargetFrequencyMin', 'TargetFrequencyMax', 'EventsPerYear', 'Duration', 'MinSpell', 
                               'FlowThresholdMin', 'FlowThresholdMax', 'MaxInter-event', 'WithinEventGapTolerance', 'WeirpoolGauge', 'FlowLevelVolume', 
@@ -101,7 +101,7 @@ def get_EWR_table(file_path:str = None) -> dict:
     return okay_EWRs, bad_EWRs
 
 @cached(cache=TTLCache(maxsize=1024, ttl=1800))
-def map_gauge_to_catchment(my_url:str = 'https://az3mdbastg001.blob.core.windows.net/mdba-public-data/NSWEWR_LIVE_DEV.csv') -> dict:
+def map_gauge_to_catchment(my_url:str = "py_ewr/parameter_metadata/NSWEWR.csv") -> dict:
     ''' Allocates all the locations in the ewr table with catchments, as indicated by the
     first three numbers for each gauge 
     
@@ -279,9 +279,9 @@ def get_level_gauges() -> tuple:
     levelGauges = menindeeGauges + lachlanGauges
     
     weirpoolGauges = {'414203': '414209', 
-                      '425010': '4260501', 
-                      '4260507': '4260508',
-                      '4260505': '4260506'}
+                      '425010': 'A4260501',
+                      'A4260507': 'A4260508',
+                      'A4260505': 'A4260506'}
     
     return levelGauges, weirpoolGauges
 
